from transformers import pipeline
from transformers import AutoTokenizer, AutoModelWithLMHead

def get_sentiment(text):
    classifier = pipeline('sentiment-analysis')
    sentiment = classifier('We are very happy to introduce pipeline to the transformers repository.')
    return sentiment

# sentiment = get_sentiment("Robots are the best")
# print(sentiment)

def get_emotion(text):
    tokenizer = AutoTokenizer.from_pretrained("mrm8488/t5-base-finetuned-emotion")
    model = AutoModelWithLMHead.from_pretrained("mrm8488/t5-base-finetuned-emotion")
    input_ids = tokenizer.encode(text + '</s>', return_tensors='pt')
    output = model.generate(input_ids=input_ids,
               max_length=2)
    dec = [tokenizer.decode(ids) for ids in output]
    label = dec[0]
    return label
  
# emotion = get_emotion("i feel as if i havent blogged in ages are at least truly blogged i am doing an update cute") # Output: 'joy'
# print(emotion)
# emotion2 = get_emotion("i have a feeling i kinda lost my best friend") # Output: 'sadness'
# print(emotion2)


from transformers import pipeline
from tmh.text.translate import translate_between_languages


models = {
    'gpt-j': 'EleutherAI/gpt-neo-2.7B',
    "swedish-gpt": "birgermoell/swedish-gpt",
    "swe-gpt-wiki": "flax-community/swe-gpt-wiki",
    "gpt2": "huggingface/gpt2",
    "gpt2-medium": "huggingface/gpt2-medium",
    "gpt2-large": "huggingface/gpt2-large",
    "gpt2-xl": "huggingface/gpt2-xl",
    "gpt2-xlm": "huggingface/gpt2-xlm",
    "gpt2-xlm-mlm": "huggingface/gpt2-xlm-mlm",
}

def generate_text(model='birgermoell/swedish-gpt', prompt="AI har möjligheten att", max_length=200, temperature=0.9):
    generator = pipeline('text-generation', model=model)
    output = generator(prompt, do_sample=True, temperature=temperature, max_length=max_length)
    return output[0]['generated_text']

def list_models():
    return models

def translate_and_generate(swedish_short_text, max_length=250, temperature=0.9):

    english_short_text = translate_between_languages(swedish_short_text, "Helsinki-NLP/opus-mt-sv-en")
    # print("the long english text is", english_long_text)

    english_generation = generate_text(model='EleutherAI/gpt-neo-2.7B', prompt=english_short_text, max_length=max_length, temperature=temperature)
    # print("the english summary is", english_summary)

    swedish_generation = translate_between_languages(english_generation, "Helsinki-NLP/opus-mt-en-sv")
    # print("the swedish summary is", swedish_summary)
    return swedish_generation

# generate text with translation

# output = generate_text(model='birgermoell/swedish-gpt', prompt="AI har möjligheten att", min_length=250)
# output = generate_text(model='EleutherAI/gpt-neo-2.7B', prompt="EleutherAI has", min_length=150)
# print(output)
# print(list_models())

# output = translate_and_generate('Artificiell intelligens har möjligheten att', 500)
# print(output)

# text = generate_text(prompt="Kärlek är", max_length=250, temperature=0.9)
# print("Swedish", text)
# text2 = translate_and_generate('Kärlek är', max_length=250, temperature=0.9)
# print("Translated", text2)

# text = generate_text(prompt="Kärlek är", max_length=250, temperature=0.9)
# print("Swedish", text)


import requests
import json
import random

def get_random_integer(number):
    """
    This function takes in a number and returns a random integer between 0 and that number
    """
    return random.randint(0, number)

def get_meme(keyword):
    """
    This function takes in a keyword and returns a url of a meme
    """
    url = "http://api.giphy.com/v1/gifs/search?q=" + keyword + "&api_key=dc6zaTOxFJmzC&limit=10"
    response = requests.get(url)
    data = json.loads(response.text)
    
    return data['data'][get_random_integer(10)]['images']['original']['url']


meme = get_meme("turtles")
print(meme)


from dp.phonemizer import Phonemizer
from dp.model import model, predictor
import torch
import argparse

# this model needs a download of the model.
## Please download this model and put it in the following folder.
## https://public-asai-dl-models.s3.eu-central-1.amazonaws.com/DeepPhonemizer/en_us_cmudict_ipa_forward.pt
## The model assumes that the model is stored in the current folder.


parser = argparse.ArgumentParser(description='Phonemize Swedish or English text')
parser.add_argument("--model_path", metavar='M', type=str, nargs='?', default='./en_us_cmudict_ipa_forward.pt',
                    help='The model path to your saved checkpoint')
parser.add_argument('--language', metavar='L', type=str, default='English',
                    help='The language you would like to phonemize')
parser.add_argument('--stress', nargs='?', metavar='S', type=str, default="n",
                    help='Indicate whether your model includes stress mark (y or n)')
parser.add_argument('--text', metavar="T", type=str,
                    help="The text you would like to phonemize formatted as a single string")
parser.add_argument('--save_to_file', nargs='?', metavar='F', type=str, 
                    help='If you want to save the phonemized text, write the filepath')
args = parser.parse_args()

def str_to_bool(string):
    if string == "y":
        return True
    if string == "n":
        return False

model_path = args.model_path
language = args.language
stress = str_to_bool(args.stress)
text = args.text
save_path = args.save_to_file

def get_swedish_phonemes(text, model_path, stress_marks=False):
    checkpoint = model_path
    if stress_marks:
        transformer = model.ForwardTransformer(55, 55, d_fft=1024, d_model=512, dropout=0.1, heads=4, layers=6)
    else:
        transformer = model.ForwardTransformer(55, 52, d_fft=1024, d_model=512, dropout=0.1, heads=4, layers=6)
    device = torch.device('cpu')
    checkpoint = torch.load(checkpoint, map_location=device)
    transformer.load_state_dict(checkpoint['model'])
    preprocessor = checkpoint['preprocessor']
    pred = predictor.Predictor(transformer, preprocessor)
    phoneme_dict = checkpoint['phoneme_dict']
    phonemize = Phonemizer(pred, phoneme_dict)
    phonemes = phonemize(text, 'se')
    result = ''
    for i, ph in enumerate(phonemes):
        if ph == ' ':
            result = result + '_ '
        if ph not in '23:_ ':
            result = result + ph + ' '
        elif ph != ' ':
            result = result[:-1] + ph + ' ' 
    return result

def get_phonemes(text, model_checkpoint='./en_us_cmudict_ipa_forward.pt', language='English', stress_marks=False):   
    if language == 'English':
        phonemizer = Phonemizer.from_checkpoint(model_checkpoint)
        phonemes = phonemizer(text, lang='en_us')
        return phonemes
    if language == 'Swedish' and stress_marks:
        return get_swedish_phonemes(text, model_checkpoint, stress_marks=True)
    else:
        return get_swedish_phonemes(text, model_checkpoint)

if __name__=='__main__':
    if save_path:
        phones = get_phonemes(text, model_path, language, stress)
        with open(save_path, "a", encoding="utf-8") as outfile:
            outfile.write(phones + '\n')

    else:
        print(get_phonemes(text, model_path, language, stress))




from lib2to3.pytree import convert
from xdrlib import ConversionError
import torchaudio
import torch
from itertools import groupby
from transformers import Wav2Vec2Processor, Wav2Vec2ForCTC
from transformers import HubertForSequenceClassification, Wav2Vec2FeatureExtractor
from transformers import pipeline
import librosa

from speechbrain.pretrained import EncoderClassifier
from typing import Any
import soundfile as sf
import os
import numpy as np

# from language_files import get_model

class ConversionError(Exception):
    pass

language_dict = {
    "Swedish": "KBLab/wav2vec2-large-voxrex-swedish",
    "English": "jonatasgrosman/wav2vec2-large-xlsr-53-english",
    "Russian": "jonatasgrosman/wav2vec2-large-xlsr-53-russian",
    "Spanish": "facebook/wav2vec2-large-xlsr-53-spanish",
    "French": "facebook/wav2vec2-large-xlsr-53-french",
    "Persian": "m3hrdadfi/wav2vec2-large-xlsr-persian",
    "Dutch": "facebook/wav2vec2-large-xlsr-53-dutch",
    "Portugese": "facebook/wav2vec2-large-xlsr-53-portuguese",
    "Chinese": "jonatasgrosman/wav2vec2-large-xlsr-53-chinese-zh-cn",
    "German": "jonatasgrosman/wav2vec2-large-xlsr-53-german",
    "Greek": "lighteternal/wav2vec2-large-xlsr-53-greek",
    "Hindi": "theainerd/Wav2Vec2-large-xlsr-hindi",
    "Italian": "jonatasgrosman/wav2vec2-large-xlsr-53-italian",
    "Turkish": "cahya/wav2vec2-base-turkish-artificial-cv",
    "Vietnamese": "leduytan93/Fine-Tune-XLSR-Wav2Vec2-Speech2Text-Vietnamese",
    "Catalan": "ccoreilly/wav2vec2-large-100k-voxpopuli-catala",
    "Japanese": "vumichien/wav2vec2-large-xlsr-japanese-hiragana",
    "Tamil": "vumichien/wav2vec2-large-xlsr-japanese-hiragana",
    "Indonesian": "indonesian-nlp/wav2vec2-large-xlsr-indonesian",
    "Dhivevi": "shahukareem/wav2vec2-large-xlsr-53-dhivehi",
    "Polish": "jonatasgrosman/wav2vec2-large-xlsr-53-polish",
    "Thai": "sakares/wav2vec2-large-xlsr-thai-demo",
    "Hebrew": "imvladikon/wav2vec2-large-xlsr-53-hebrew",
    "Mongolian": "sammy786/wav2vec2-large-xlsr-mongolian",
    "Czech": "arampacha/wav2vec2-large-xlsr-czech",
    "Icelandic": "m3hrdadfi/wav2vec2-large-xlsr-icelandic",
    "Irish": "jimregan/wav2vec2-large-xlsr-irish-basic",
    "Kinyarwanda": "lucio/wav2vec2-large-xlsr-kinyarwanda",
    "Lithuanian": "DeividasM/wav2vec2-large-xlsr-53-lithuanian",
    "Hungarian": "jonatasgrosman/wav2vec2-large-xlsr-53-hungarian",
    "Finnish": "aapot/wav2vec2-large-xlsr-53-finnish"
    }

# to do 
# chech language
# enable batch mode
def extract_speaker_embedding(audio_path):
    classifier = EncoderClassifier.from_hparams(source="speechbrain/spkrec-xvect-voxceleb", savedir="pretrained_models/spkrec-xvect-voxceleb")
    signal, fs =torchaudio.load(audio_path)
    embeddings = classifier.encode_batch(signal)
    # print(embeddings)
    return embeddings

def change_sample_rate(audio_path, new_sample_rate=16000):
    audio_to_resample, sr = librosa.load(audio_path)
    resampled_audio = librosa.resample(audio_to_resample, sr, new_sample_rate)
    resampled_tensor = torch.tensor([resampled_audio])
    return resampled_tensor

def classify_emotion(audio_path):
    model = HubertForSequenceClassification.from_pretrained("superb/hubert-large-superb-er")
    feature_extractor = Wav2Vec2FeatureExtractor.from_pretrained("superb/hubert-large-superb-er")
    speech, _ = librosa.load(audio_path, sr=16000, mono=True)

    inputs = feature_extractor(speech, sampling_rate=16000, padding=True, return_tensors="pt")

    logits = model(**inputs).logits
    predicted_ids = torch.argmax(logits, dim=-1)
    labels = [model.config.id2label[_id] for _id in predicted_ids.tolist()]
    # print(labels)
    return(labels)

def classify_language(audio_path):
    classifier = EncoderClassifier.from_hparams(source="speechbrain/lang-id-commonlanguage_ecapa", savedir="pretrained_models/lang-id-commonlanguage_ecapa")
    out_prob, score, index, text_lab = classifier.classify_file(audio_path)
    return(text_lab[0])

def convert_to_wav(audio_path):
    path = audio_path.split("/")
    filename = path[-1].split(".")[0]
    audio, sr = librosa.load(audio_path)
    wav_path = f'{filename}.wav'
    sf.write(wav_path, audio, sr, subtype='PCM_24')
    return wav_path

def get_speech_rate_time_stamps(time_stamps, downsample=320, sample_rate=16000):

    utterances = len(time_stamps[0])
    start_time = time_stamps[0][0]['start_offset']
    end_time = time_stamps[0][utterances-1]['end_offset']
    duration = end_time - start_time
    
    speech_rate = ((duration / utterances) * downsample) / sample_rate
    
    return speech_rate

def calculate_variance(data):
    n = len(data)
    mean = sum(data) / n
    # Square deviations
    deviations = [(x - mean) ** 2 for x in data]
    # Variance
    variance = sum(deviations) / n
    return variance

def get_speech_rate_variability(time_stamps, type='char', downsample=320, sample_rate=16000 ):
    base = downsample / sample_rate
    token_durations = {}

    for time_stamp in time_stamps[0]:

        start_time = round(time_stamp['start_offset']*base, 2)
        end_time = round(time_stamp['end_offset']*base, 2)
        char = time_stamp[type]
        duration = end_time - start_time

        if char not in token_durations:
            token_durations[char] = []

        token_durations[char].append(duration)

    averages = dict()
    stds = dict()
    variances = dict()

    for token, durations in token_durations.items():
        average = np.sum(durations) / len(durations)
        std = np.std(durations)
        # print("the tokens are", token)
        # print("the durations are", durations)
        # print("the average is", average)
        variance = calculate_variance(durations)
        averages[token] = average
        stds[token] = std
        variances[token] = variance
    
    return averages, stds, variances

def transcribe_from_audio_path(audio_path, language='Swedish', check_language=False, classify_emotion=False, model="", output_word_offsets=False):
    converted = False
    if audio_path[-4:] != ".wav":
        try:
            audio_path = convert_to_wav(audio_path)
            converted = True
        except:
            raise ConversionError(f"Could not convert {audio_path} to wav")
    
    waveform, sample_rate = torchaudio.load(audio_path)
    if sample_rate != 16000:
        #resample to 16000 Hz
        waveform = change_sample_rate(audio_path)
        sample_rate = 16000

    if converted:
        os.remove(audio_path)

    if check_language:
        language = classify_language(audio_path)
        # print("the language is", language)
        try:
            model_id = language_dict[language]
        except KeyError:
            print("No language model found for %s. Defaulting to KBLab/wav2vec2-large-voxrex-swedish unless another model was specified." %language)
            model_id = "KBLab/wav2vec2-large-voxrex-swedish"
    else:
        try:
            model_id = language_dict[language]
        except KeyError:
            print("No language model found for %s. Defaulting to KBLab/wav2vec2-large-voxrex-swedish unless another model was specified." %language)
            model_id = "KBLab/wav2vec2-large-voxrex-swedish"

    if model:
        model_id = model

    processor = Wav2Vec2Processor.from_pretrained(model_id)
    model = Wav2Vec2ForCTC.from_pretrained(model_id)
    with torch.no_grad():
        logits = model(waveform).logits
    pred_ids = torch.argmax(logits, dim=-1)

    if output_word_offsets:
        outputs = processor.batch_decode(pred_ids, output_word_offsets=output_word_offsets)
        transcription = outputs["text"][0]
        token_time_stamps = outputs[1]
        speech_rate = get_speech_rate_time_stamps(token_time_stamps)
        averages, stds, variances = get_speech_rate_variability(token_time_stamps, type="char")
        word_time_stamps = outputs[2]
        return {
            "transcription": transcription,
            "speech_rate": speech_rate,
            "averages": averages,
            "standard_deviations": stds,
            "variances": variances
        }
    else:
        transcription = processor.batch_decode(pred_ids)
        return transcription[0]



file_path = "/home/bmoell/tmh/tmh/test.wav"
output = transcribe_from_audio_path(file_path, "English", output_word_offsets=True)
print("the output is", output)
# transcription = "Det visste i varje fall näsan."
# print("the transcription is", transcription)


from espnet2.bin.tts_inference import Text2Speech
    
model = Text2Speech.from_pretrained("julien-c/ljspeech_tts_train_tacotron2_raw_phn_tacotron_g2p_en_no_space_train")

speech, *_ = model("text to generate speech from")
print(speech)


from dp.phonemizer import Phonemizer

def try_it_out():
    checkpoint_path = '/Users/bmoell/Code/tmh/models/swedish_phonemizer.pt'
    phonemizer = Phonemizer.from_checkpoint(checkpoint_path)
    text = 'varför händer det här'
    result = phonemizer.phonemise_list([text], lang='se')
    print(result.phonemes)
    for text, pred in result.predictions.items():
        tokens, probs = pred.phoneme_tokens, pred.token_probs
        for o, p in zip(tokens, probs):
            print(f'{o} {p}')
        tokens = ''.join(tokens)
        print(f'{text} | {tokens} | {pred.confidence}')

try_it_out()


# from vad import extract_speak_segments
from librosa.core.spectrum import power_to_db
import torch
from overlap import overlap_detection
from transcribe import extract_speaker_embedding
import numpy as np

from sklearn.preprocessing import StandardScaler

from pydub import AudioSegment
from audio_embeddings import get_audio_embeddings

from pyannote.audio import Inference
from pyannote.audio.pipelines import VoiceActivityDetection
from transformers import Wav2Vec2Processor, Wav2Vec2Model
from sklearn.mixture import GaussianMixture
from sklearn.cluster import KMeans, AgglomerativeClustering, SpectralClustering, MiniBatchKMeans
from pyannote.core import Segment, Timeline
import torchaudio
from speechbrain.pretrained import EncoderClassifier
# from language_files import get_model

pipeline = VoiceActivityDetection(segmentation="pyannote/segmentation")

# speaker embedding
inference = Inference("pyannote/embedding", window="whole")

HYPER_PARAMETERS = {
  # onset/offset activation thresholds
  "onset": 0.5, "offset": 0.5,
  # remove speech regions shorter than that many seconds.
  "min_duration_on": 0.0,
  # fill non-speech regions shorter than that many seconds.
  "min_duration_off": 0.0
}

def extract_speak_segments(audio_path):
    pipeline.instantiate(HYPER_PARAMETERS)    
    vad = pipeline(audio_path)
    # print("extracting speaker segments")
    # print(vad)
    return(vad.for_json())


def classify_speakers_based_on_embeddings(X):
    X = StandardScaler().fit_transform(X)
    
    estimator=GaussianMixture(n_components=2)
    y = estimator.fit_predict(X)
    return y


def extract_speaker_embedding(signal, audio):

    audio_file = torch.tensor(audio[signal.start*1000 : signal.end * 1000].get_array_of_samples())
    #embeddings = classifier.encode_batch(audio_file)
    embeddings = inference( {"waveform": audio_file.unsqueeze(0).float(), "sample_rate": 44100} )

    # print(embeddings)
    return embeddings
    


def create_speaker_files_from_audio_path(audio_path):

    from pyannote.audio import Pipeline
    pipeline = Pipeline.from_pretrained("pyannote/speaker-diarization")

    # apply pretrained pipeline
    audio = AudioSegment.from_wav(audio_path)

    diarization = pipeline(audio_path)

    speaker1= AudioSegment.empty()
    speaker2 = AudioSegment.empty()

    # print the result
    for turn, _, speaker in diarization.itertracks(yield_label=True):
        print(f"start={turn.start:.1f}s stop={turn.end:.1f}s speaker_{speaker}")
        current_audio_segment = audio[turn.start*1000:turn.end*1000]
        if speaker == "SPEAKER_00":
            print("speaker1", speaker)
            speaker1+=current_audio_segment
        elif speaker == "SPEAKER_01":
            print("speaker2", speaker2)
            speaker2+=current_audio_segment

        speaker1.export("speaker1.wav", format="wav")
        speaker2.export("speaker2.wav", format="wav")


def wavlm_speaker_diarization(audio_path):
    from transformers import Wav2Vec2FeatureExtractor, UniSpeechSatForAudioFrameClassification
    import torch

    audio_file = torchaudio.load(audio_path)

    feature_extractor = Wav2Vec2FeatureExtractor.from_pretrained('microsoft/wavlm-base-sd')
    model = UniSpeechSatForAudioFrameClassification.from_pretrained('microsoft/wavlm-base-sd')



    # audio file is decoded on the fly
    inputs = feature_extractor(audio_file, return_tensors="pt")
    logits = model(**inputs).logits
    probabilities = torch.sigmoid(logits[0])

    # labels is a one-hot array of shape (num_frames, num_speakers)
    labels = (probabilities > 0.5).long()
    print(labels)


def create_speaker_files_from_audio_path_old(audio_path):
    """
    Creates speaker files from the audio file at audio_path by splitting based on speaker.

    If the 'save_to_file' parameter is set to a file name, the
    results will also be written to file. 

    The srt format can be used to create accurately timed subtitles 
    for videos. If 'audio_path' is the audio track of a video file, 
    a video player like VLC will create those subtitles automatically, 
    given a file in the srt format.
    """
    
    audio = AudioSegment.from_wav(audio_path)

    if audio.frame_rate != 16000:
        ## change sample rate to 16000
        waveform = audio.set_frame_rate(16_000)

    
    # first we get all the overlaps for each speaker in the file
    overlaps = overlap_detection(audio_path)
    print(overlaps)

    # removing overlaps
    for overlapping_segment in overlaps:
        print(overlapping_segment  )


    starting_time = 0
    without_overlaps_audio = AudioSegment.empty()
    for overlaping_segments in overlaps['content']:
        segment = overlaping_segments['segment']
        current_segment_start_time = segment['start'] * 1000
        current_segment_end_time = segment['end'] * 1000
        current_segment = audio[starting_time: current_segment_start_time]
        starting_time = current_segment_end_time
        
        without_overlaps_audio += current_segment
    
    without_overlaps_audio.export("temp.wav", format="wav")
    
    segments = extract_speak_segments('temp.wav')
 
    embeddings = []
        
    inference = Inference("pyannote/embedding", window="whole")
    
    timeline = Timeline()
    count_short = 0
    for segment in segments['content']:
        current_segment_time = segment['segment']
        start_time = current_segment_time['start']
        end_time = current_segment_time['end']

        if end_time - start_time < 1:
            count_short += 1
            print(str(count_short) + ' short segments')
            continue
        current_segment = Segment(start_time, end_time)
        print('Long segment: ', current_segment)
        #embedding = inference.crop("temp.wav", current_segment)
        embedding = extract_speaker_embedding(current_segment, audio)
        
        
        timeline.add(current_segment)
        embeddings.append(embedding)
    
    embeddings = np.vstack(embeddings)
    
    y = classify_speakers_based_on_embeddings(embeddings)

    speaker1= AudioSegment.empty()
    speaker2 = AudioSegment.empty()

    for i, segment in enumerate(timeline):
        current_audio_segment = audio[segment.start*1000:segment.end*1000]
        if y[i] == 0:


            speaker1 += current_audio_segment
        else:
            speaker2 += current_audio_segment

    
    speaker1.export("speaker1_old.wav", format="wav")
    speaker2.export("speaker2_old.wav", format="wav")
    
            

    return embeddings

# model_ids = {
#         "hubert": "facebook/hubert-large-ls960-ft",
#         "wav2vec2": "facebook/wav2vec2-base-960h",
# }

# processor = Wav2Vec2Processor.from_pretrained("facebook/wav2vec2-base-960h")
# model = Wav2Vec2Model.from_pretrained("facebook/wav2vec2-base-960h")

# def get_audio_embeddings(audio_array, sample_rate, model_id="facebook/wav2vec2-base-960h"):
#     input_values = processor(audio_array, sampling_rate=sample_rate, return_tensors="pt").input_values
#     output = model(**input_values)
#     hidden_states = output.last_hidden_state
#     return hidden_states


def time_format( t ) :
    """
    Produces the time format expected by the srt format,
    given a parameter t representing a number of seconds.
    """
    hours = int(t/3600)
    minutes = int((t-hours*3600)/60)
    seconds = int(t-hours*3600-minutes*60)
    fraction = int((t%1)*100)
    return str(hours) + ":" + str(minutes) + ":" + str(seconds) + "." + str(fraction)


file_path = "/home/bmoell/tmh/tmh/test.wav"
wavlm_speaker_diarization(file_path)



from transformers import Wav2Vec2Processor, Wav2Vec2Model
import soundfile as sf
import numpy as np

model_ids = {
        "hubert": "facebook/hubert-large-ls960-ft",
        "wav2vec2": "facebook/wav2vec2-base-960h",
}

def get_audio_embeddings(audio_path, model_id="facebook/hubert-large-ls960-ft"):
        processor = Wav2Vec2Processor.from_pretrained(model_id)
        model = Wav2Vec2Model.from_pretrained(model_id)
        y, sample_rate = sf.read(audio_path)
        
        with torch.no_grad():
            
            input_values = processor(y, sampling_rate=sample_rate, return_tensors="pt", padding=True).input_values
            hidden_states = model(input_values).last_hidden_state
            return hidden_states

# embed =audio_embeddings = get_audio_embeddings('/Users/bmoell/Code/test_tanscribe/sv.wav')
# print(embed)




language_dict = {
    "Swedish": "KBLab/wav2vec2-large-voxrex-swedish",
    "English": "jonatasgrosman/wav2vec2-large-xlsr-53-english",
    "Russian": "jonatasgrosman/wav2vec2-large-xlsr-53-russian",
    "Spanish": "facebook/wav2vec2-large-xlsr-53-spanish",
    "French": "facebook/wav2vec2-large-xlsr-53-french",
    "Persian": "m3hrdadfi/wav2vec2-large-xlsr-persian",
    "Dutch": "facebook/wav2vec2-large-xlsr-53-dutch",
    "Portugese": "facebook/wav2vec2-large-xlsr-53-portuguese",
    "Chinese": "jonatasgrosman/wav2vec2-large-xlsr-53-chinese-zh-cn",
    "German": "jonatasgrosman/wav2vec2-large-xlsr-53-german",
    "Greek": "lighteternal/wav2vec2-large-xlsr-53-greek",
    "Hindi": "theainerd/Wav2Vec2-large-xlsr-hindi",
    "Italian": "jonatasgrosman/wav2vec2-large-xlsr-53-italian",
    "Turkish": "cahya/wav2vec2-base-turkish-artificial-cv",
    "Vietnamese": "leduytan93/Fine-Tune-XLSR-Wav2Vec2-Speech2Text-Vietnamese",
    "Catalan": "ccoreilly/wav2vec2-large-100k-voxpopuli-catala",
    "Japanese": "vumichien/wav2vec2-large-xlsr-japanese-hiragana",
    "Tamil": "vumichien/wav2vec2-large-xlsr-japanese-hiragana",
    "Indonesian": "indonesian-nlp/wav2vec2-large-xlsr-indonesian",
    "Dhivevi": "shahukareem/wav2vec2-large-xlsr-53-dhivehi",
    "Polish": "jonatasgrosman/wav2vec2-large-xlsr-53-polish",
    "Thai": "sakares/wav2vec2-large-xlsr-thai-demo",
    "Hebrew": "imvladikon/wav2vec2-large-xlsr-53-hebrew",
    "Mongolian": "sammy786/wav2vec2-large-xlsr-mongolian",
    "Czech": "arampacha/wav2vec2-large-xlsr-czech",
    "Icelandic": "m3hrdadfi/wav2vec2-large-xlsr-icelandic",
    "Irish": "jimregan/wav2vec2-large-xlsr-irish-basic",
    "Kinyarwanda": "lucio/wav2vec2-large-xlsr-kinyarwanda",
    "Lithuanian": "DeividasM/wav2vec2-large-xlsr-53-lithuanian",
    "Hungarian": "jonatasgrosman/wav2vec2-large-xlsr-53-hungarian",
    "Finnish": "aapot/wav2vec2-large-xlsr-53-finnish"
    }

def get_model(language):
    return language_dict[language]

language_list = "Arabic, Basque, Breton, Catalan, Chinese_China, Chinese_Hongkong, Chinese_Taiwan, Chuvash, Czech, Dhivehi, Dutch, English, Esperanto, Estonian, French, Frisian, Georgian, German, Greek, Hakha_Chin, Indonesian, Interlingua, Italian, Japanese, Kabyle, Kinyarwanda, Kyrgyz, Latvian, Maltese, Mangolian, Persian, Polish, Portuguese, Romanian, Romansh_Sursilvan, Russian, Sakha, Slovenian, Spanish, Swedish, Tamil, Tatar, Turkish, Ukranian, Welsh"



from transformers import pipeline

models = {
    'kb-ner': 'KB/bert-base-swedish-cased-ner',
    "swedish-wikiann": "birgermoell/ner-swedish-wikiann"
}

def named_entity_recognition(text, model='KB/bert-base-swedish-cased-ner', tokenizer='KB/bert-base-swedish-cased-ner'):
    """
    Named entity recognition
    """
    # Load the model
    ner = pipeline('ner', model=model, tokenizer=tokenizer)
    # Get the result
    result = ner(text)
    # Return the resultß
    return result

# ner = named_entity_recognition('KTH är ett universitet i Stockholm')
# print(ner)


from transformers import pipeline
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM
from transformers import MarianMTModel, MarianTokenizer
from transformers import PegasusForConditionalGeneration, PegasusTokenizer
ARTICLE = """ New York (CNN)When Liana Barrientos was 23 years old, she got married in Westchester County, New York.
 A year later, she got married again in Westchester County, but to a different man and without divorcing her first husband.
 Only 18 days after that marriage, she got hitched yet again. Then, Barrientos declared "I do" five more times, sometimes only within two weeks of each other.
 In 2010, she married once more, this time in the Bronx. In an application for a marriage license, she stated it was her "first and only" marriage.
 Barrientos, now 39, is facing two criminal counts of "offering a false instrument for filing in the first degree," referring to her false statements on the
 2010 marriage license application, according to court documents.
 Prosecutors said the marriages were part of an immigration scam.
 On Friday, she pleaded not guilty at State Supreme Court in the Bronx, according to her attorney, Christopher Wright, who declined to comment further.
 After leaving court, Barrientos was arrested and charged with theft of service and criminal trespass for allegedly sneaking into the New York subway through an emergency exit, said Detective
 Annette Markowski, a police spokeswoman. In total, Barrientos has been married 10 times, with nine of her marriages occurring between 1999 and 2002.
 All occurred either in Westchester County, Long Island, New Jersey or the Bronx. She is believed to still be married to four men, and at one time, she was married to eight men at once, prosecutors say.
 Prosecutors said the immigration scam involved some of her husbands, who filed for permanent residence status shortly after the marriages.
 Any divorces happened only after such filings were approved. It was unclear whether any of the men will be prosecuted.
 The case was referred to the Bronx District Attorney\'s Office by Immigration and Customs Enforcement and the Department of Homeland Security\'s
 Investigation Division. Seven of the men are from so-called "red-flagged" countries, including Egypt, Turkey, Georgia, Pakistan and Mali.
 Her eighth husband, Rashid Rajput, was deported in 2006 to his native Pakistan after an investigation by the Joint Terrorism Task Force.
 If convicted, Barrientos faces up to four years in prison.  Her next court appearance is scheduled for May 18.
 """

# print(summarizer(ARTICLE, max_length=130, min_length=30, do_sample=False))

def get_summary(text, model="t5-base"):
    summarizer = pipeline("summarization")
    result =summarizer(text, max_length=130, min_length=30, do_sample=False)
    return result


def translate_between_languages(text, model):

    tokenizer = AutoTokenizer.from_pretrained(model)
    model = AutoModelForSeq2SeqLM.from_pretrained(model)

    translated = model.generate(**tokenizer(text, return_tensors="pt", padding=True))
    output = [tokenizer.decode(t, skip_special_tokens=True) for t in translated]
    # print(output)
    return output[0]

def pegasus_summary(text):
  
    tokenizer = AutoTokenizer.from_pretrained("google/pegasus-xsum")

    model = AutoModelForSeq2SeqLM.from_pretrained("google/pegasus-xsum")

    batch = tokenizer(text, truncation=True, padding='longest', return_tensors="pt")
    translated = model.generate(**batch)
    tgt_text = tokenizer.batch_decode(translated, skip_special_tokens=True)
    return tgt_text[0]

def translate_and_summarize(swedish_long_text):

    english_long_text = translate_between_languages(swedish_long_text, "Helsinki-NLP/opus-mt-sv-en")
    # print("the long english text is", english_long_text)

    english_summary = pegasus_summary(english_long_text)
    # print("the english summary is", english_summary)

    swedish_summary = translate_between_languages(english_summary, "Helsinki-NLP/opus-mt-en-sv")
    # print("the swedish summary is", swedish_summary)
    return swedish_summary

   

# summary = pegasus_summary(ARTICLE)
# print(summary)
# sv_text = "Albert Einstein var son till Hermann och Pauline Einstein, vilka var icke-religiösa judar och tillhörde medelklassen. Fadern var försäljare och drev senare en elektroteknisk fabrik. Familjen bosatte sig 1880 i München där Einstein gick i en katolsk skola. Mängder av myter cirkulerar om Albert Einsteins person. En av dessa är att han som barn skulle ha haft svårigheter med matematik, vilket dock motsägs av hans utmärkta betyg i ämnet.[15] Han nämnde ofta senare att han inte trivdes i skolan på grund av dess pedagogik. Att Albert Einstein skulle vara släkt med musikvetaren Alfred Einstein är ett, ofta framfört, obevisat påstående. Alfred Einsteins dotter Eva har framhållit att något sådant släktskap inte existerar."

# swedish_summary = translate_and_summarize(sv_text)
# print(swedish_summary)
# sum = get_summary(ARTICLE)
# print(sum)